// Proportial teapot (based on  the textured teapot)

// Are you annoyed by the scene getting weird if you rescale the window? This demo shows how you use the "reshape" callback
// to adjust the projection. I am adjusting the frustum() call but you can use perspective() in a similar way. It is also
// possible to query the window size and adjusting for every frame but that is a bit wasteful.

// gcc proportionalteapot.cpp -o proportionalteapot ../common/*.c ../common/Linux/*.c -I../common -I../common/Linux -lGL -lX11 -DGL_GLEXT_PROTOTYPES -lm

#define MAIN
#include "GL_utilities.h"
#include "MicroGlut.h"
#include "VectorUtils4.h"
#include "LittleOBJLoader.h"
#include "LoadTGA.h"
//uses framework Cocoa
//uses framework OpenGL

#define near 1.0
#define far 30.0
#define right 0.5
#define left -0.5
#define top 0.5
#define bottom -0.5

// vertex array object
unsigned int bunnyVertexArrayObjID;
Model *m;
// Reference to shader program
GLuint program;
GLuint tex;

void init(void)
{
	// GL inits
	glClearColor(0.2,0.2,0.5,0);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);
	glCullFace(GL_BACK);
	printError("GL inits");

	// Load and compile shader
	program = loadShaders("tex.vert", "tex.frag");
	glUseProgram(program);
	printError("init shader");
	
	// Upload geometry to the GPU:
	m = LoadModel("teapot.obj");
		
	// End of upload of geometry

	mat4 camMatrix = lookAt(0,13,-15, 0,5,0, 0,1,0);
	glUniformMatrix4fv(glGetUniformLocation(program, "camMatrix"), 1, GL_TRUE, camMatrix.m);
	mat4 projectionMatrix = frustum(left, right, bottom, top, near, far);
	glUniformMatrix4fv(glGetUniformLocation(program, "projMatrix"), 1, GL_TRUE, projectionMatrix.m);
	
	printError("init arrays");
	
	glUniform1i(glGetUniformLocation(program, "tex"), 0); // Texture unit 0
	LoadTGATextureSimple("maskros512.tga", &tex); // 5c
}

void reshape(int x, int y)
{
	mat4 projectionMatrix;
	glViewport(0, 0, x, y);
	if (x > y) // wide, limit left and right
		projectionMatrix = frustum(left * x/y, right * x/y, bottom, top, near, far);
	else // high, limit top and bottom
		projectionMatrix = frustum(left, right, bottom * y/x, top * y/x, near, far);
	glUniformMatrix4fv(glGetUniformLocation(program, "projMatrix"), 1, GL_TRUE, projectionMatrix.m);
}

GLfloat a = 0.0;

void display(void)
{
	printError("pre display");

	// clear the screen
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	a += 0.1;
	mat4 rot = Ry(a/2);
	glUniformMatrix4fv(glGetUniformLocation(program, "mdlMatrix"), 1, GL_TRUE, rot.m);
	
	DrawModel(m, program, "inPosition", "inNormal", "inTexCoord");
	
	printError("display");
	
	glutSwapBuffers();
}

int main(int argc, char *argv[])
{
	glutInit(&argc, argv);
	glutInitContextVersion(3, 2);
	glutInitDisplayMode(GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE);
	glutInitWindowSize(800, 800);
	glutCreateWindow ("Scaling example");
	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	init ();
	glutRepeatingTimer(20);
	glutMainLoop();
	exit(0);
}

