// Textured model (by default Utah Teapot)

// Variant 2, using LittleOBJLoader's abstraction of the model data as well as more VectorUtils for
// nicer matrix management. You can do it all yourself, and the other demos show how. Then you want
// to abstract and package things, like in this demo.

// gcc texturedteapot2.cpp -o texturedteapot2 ../common/*.c ../common/Linux/*.c -I../common -I../common/Linux -lGL -lX11 -DGL_GLEXT_PROTOTYPES -lm

#define MAIN
#include "GL_utilities.h"
#include "MicroGlut.h"
#include "VectorUtils4.h"
#include "LittleOBJLoader.h"
#include "LoadTGA.h"
//uses framework Cocoa
//uses framework OpenGL

#define near 1.0
#define far 30.0
#define right 0.5
#define left -0.5
#define top 0.5
#define bottom -0.5

// vertex array object
unsigned int bunnyVertexArrayObjID;
Model *m;
// Reference to shader program
GLuint program;
GLuint tex;

void init(void)
{
	// GL inits
	glClearColor(0.2,0.2,0.5,0);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);
	glCullFace(GL_BACK);
	printError("GL inits");

	// Load and compile shader
	program = loadShaders("tex.vert", "tex.frag");
	glUseProgram(program);
	printError("init shader");
	
	// Upload geometry to the GPU:
	m = LoadModel("teapot.obj");
		
	// End of upload of geometry

	mat4 camMatrix = lookAt(0,13,-15, 0,5,0, 0,1,0);
	printMat4(camMatrix);
	glUniformMatrix4fv(glGetUniformLocation(program, "camMatrix"), 1, GL_TRUE, camMatrix.m);
	mat4 projectionMatrix = frustum(left, right, bottom, top, near, far);
	glUniformMatrix4fv(glGetUniformLocation(program, "projMatrix"), 1, GL_TRUE, projectionMatrix.m);
	
	printError("init arrays");
	
	glUniform1i(glGetUniformLocation(program, "tex"), 0); // Texture unit 0
	LoadTGATextureSimple("maskros512.tga", &tex); // 5c
}

GLfloat a = 0.0;

void display(void)
{
	printError("pre display");

	// clear the screen
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	a += 0.1;
	mat4 rot = Ry(a/2);
	glUniformMatrix4fv(glGetUniformLocation(program, "mdlMatrix"), 1, GL_TRUE, rot.m);
	
	DrawModel(m, program, "inPosition", "inNormal", "inTexCoord");
	
	printError("display");
	
	glutSwapBuffers();
}

int main(int argc, char *argv[])
{
	glutInit(&argc, argv);
	glutInitContextVersion(3, 2);
	glutInitDisplayMode(GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE);
	glutCreateWindow ("Texture example");
	glutDisplayFunc(display);
	init ();
	glutRepeatingTimer(20);
	glutMainLoop();
	exit(0);
}

